///
///author: linkcd
///date: 2005-08-30
///

using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;

using System.Runtime.Remoting;
using System.Runtime.Remoting.Channels;
using System.Runtime.Remoting.Channels.Tcp;

using Server;

namespace ContainerTest
{
	public partial class Client : Form
	{
		enum TestCaseType
		{
			ReadOneRow,
			ReadManyRows,
			ReadOneMasterAndManyDetail
		}
		
		public Client()
		{
			InitializeComponent();
		}

		#region Form Event Handler
		private void buttonClose_Click(object sender, EventArgs e)
		{
			this.Close();
		}

		private void buttonReset_Click(object sender, EventArgs e)
		{
			this.InitForm();
		}

		private void Client_Load(object sender, EventArgs e)
		{
			this.InitForm();

			TcpChannel channel = new TcpChannel();
			ChannelServices.RegisterChannel(channel);
		}

		private void radioButtonManyRows_CheckedChanged(object sender, EventArgs e)
		{
			this.SetCorrectCountTextbox();
		}

		private void radioButtonReadMasterAndDetail_CheckedChanged(object sender, EventArgs e)
		{
			this.SetCorrectCountTextbox();
		}

		private void radioButtonReadOneRow_CheckedChanged(object sender, EventArgs e)
		{
			this.SetCorrectCountTextbox();
		}

		private void buttonExecute_Click(object sender, EventArgs e)
		{
			//clear result
			this.ClearResultTextBoxes();

			//Get parameters from UI
			string hostAddress = this.textBoxHostAddress.Text.Trim();
			int seconds = int.Parse(this.textBoxNumberOfExecute.Text);
			int countOfRecords = int.Parse(this.textBoxCountOfFetchRecords.Text);

			TestCaseType serverType;
			if (this.radioButtonReadOneRow.Checked)
			{
				serverType = TestCaseType.ReadOneRow;
			}
			else
			{
				if (this.radioButtonManyRows.Checked)
				{
					serverType = TestCaseType.ReadManyRows;
				}
				else
				{
					serverType = TestCaseType.ReadOneMasterAndManyDetail;
				}
			}

			//GO!
			this.RunTest(seconds, hostAddress, serverType, countOfRecords);

		}
		#endregion

		#region Private Methods
		
		/// <summary>
		/// Init the data of this form 
		/// </summary>
		private void InitForm()
		{
			this.radioButtonReadOneRow.Checked = true;
			
			this.textBoxHostAddress.Text = @"127.0.0.1:8005";
			this.textBoxNumberOfExecute.Text = "10";
			this.textBoxCountOfFetchRecords.Text = "3000";

			this.ClearResultTextBoxes();
		}

		private void ClearResultTextBoxes()
		{
			this.textBoxResultOfBinaryDS.Clear();
			this.textBoxResultOfCustomClasses.Clear();
			this.textBoxResultOfXmlDS.Clear();
		}

		private void RegisterChannel(string hostAddress)
		{
			//this host address have not be registered, reisger it
			string tcpHostAddress = "tcp://" + hostAddress;

			Type typeOfDatasetServer = typeof(DatasetServer);
			Type typeOfCustomClassServer = typeof(CustomClassServer);

			if (RemotingConfiguration.IsRemotelyActivatedClientType(typeOfDatasetServer) == null)
			{
				RemotingConfiguration.RegisterActivatedClientType(typeOfDatasetServer, tcpHostAddress);
			}

			if (RemotingConfiguration.IsRemotelyActivatedClientType(typeOfCustomClassServer) == null)
			{
				RemotingConfiguration.RegisterActivatedClientType(typeOfCustomClassServer, tcpHostAddress);
			}
		}

		private void RunTest(int secondOfExecute, string hostAddress, TestCaseType testType, int countOfRecords)
		{
			this.RegisterChannel(hostAddress);

			IRemotingServer serverOfCustomClass = new CustomClassServer();
			IRemotingServer serverOfXmlDataset = new DatasetServer(false);
			IRemotingServer serverOfBinaryDataset = new DatasetServer(true);

			TestRunner runnerOfCustomeClass = new TestRunner(secondOfExecute, serverOfCustomClass);
			TestRunner runnerOfXmlDataset = new TestRunner(secondOfExecute, serverOfXmlDataset);
			TestRunner runnerOfBinaryDataset = new TestRunner(secondOfExecute, serverOfBinaryDataset);

			int resultOfCustomClass = 0;
			int resultOfXmlDataset = 0;
			int resultOfBinaryDataset = 0;

			switch (testType)
			{
				case TestCaseType.ReadOneRow:
					resultOfCustomClass = runnerOfCustomeClass.TestReadOneRow();
					resultOfXmlDataset = runnerOfXmlDataset.TestReadOneRow();
					resultOfBinaryDataset = runnerOfBinaryDataset.TestReadOneRow();
					break;
				case TestCaseType.ReadManyRows:
					resultOfCustomClass = runnerOfCustomeClass.TestReadManyRows(countOfRecords);
					resultOfXmlDataset = runnerOfXmlDataset.TestReadManyRows(countOfRecords);
					resultOfBinaryDataset = runnerOfBinaryDataset.TestReadManyRows(countOfRecords);
					break;
				case TestCaseType.ReadOneMasterAndManyDetail:
					resultOfCustomClass = runnerOfCustomeClass.TestReadOneMasterAndManyDetails(countOfRecords);
					resultOfXmlDataset = runnerOfXmlDataset.TestReadOneMasterAndManyDetails(countOfRecords);
					resultOfBinaryDataset = runnerOfBinaryDataset.TestReadOneMasterAndManyDetails(countOfRecords);
					break;
			}

			this.textBoxResultOfCustomClasses.Text = resultOfCustomClass.ToString();
			this.textBoxResultOfXmlDS.Text = resultOfXmlDataset.ToString();
			this.textBoxResultOfBinaryDS.Text = resultOfBinaryDataset.ToString();
		}


		private void SetCorrectCountTextbox()
		{
			if (this.radioButtonManyRows.Checked || this.radioButtonReadMasterAndDetail.Checked)
			{
				this.textBoxCountOfFetchRecords.Enabled = true;
			}
			else
			{
				this.textBoxCountOfFetchRecords.Enabled = false;
			}
		}

		#endregion
	}
}