///
///author: linkcd
///date: 2005-08-30
///
using System;
using System.Collections.Generic;
using System.Text;
using System.Data;

using Server;
using Entity;

namespace ContainerTest
{
	class TestRunner
	{
		private int _seconds;
		private IRemotingServer _server;
		private System.Diagnostics.Stopwatch _sw = new System.Diagnostics.Stopwatch();

		public TestRunner(int second, IRemotingServer server)
		{
			this._seconds = second;
			this._server = server;
		}

		private int GetRandomId()
		{
			return 500;
		}

		private void TimeStart()
		{
			_sw.Reset();
			_sw.Start();
		}

		private bool IsTimeUp()
		{
			return _sw.Elapsed.Seconds > this._seconds;
		}

		public int TestReadOneRow()
		{
			int count = 0;
			this.TimeStart();
			while (!this.IsTimeUp())
			{
				object data = _server.FetchOrder(this.GetRandomId());

				this.ConsumptionOneRow(data);
				
				count++;
			}

			return count;
		}

		private void ConsumptionOneRow(object data)
		{
			//test return only one row
			DataSet ds = data as DataSet;
			if (ds != null)
			{
				System.Diagnostics.Trace.Assert(ds.Tables[(int)OrderTables.Orders].Rows.Count == 1, "wrong data count");
				DataRow anOrder = ds.Tables[(int)OrderTables.Orders].Rows[0];
				int id = (int)anOrder[(int)OrderColumns.Id];
				int customId = (int)anOrder[(int)OrderColumns.CustomerId];
				DateTime orderDate = (DateTime)anOrder[(int)OrderColumns.OrderDate];
			}

			Order myOrder = data as Order;
			if (myOrder != null)
			{
				System.Diagnostics.Trace.Assert(myOrder.Lines.Count == 0);
				int id = myOrder.Id;
				int customId = myOrder.CustomerId;
				DateTime orderDate = myOrder.OrderDate;
			}
		}

		public int TestReadManyRows(int countOfRecords)
		{
			int count = 0;
			this.TimeStart();
			while (!this.IsTimeUp())
			{
				object data = _server.FetchOrderLines(this.GetRandomId(), countOfRecords );

				////consumption the returned data
				this.ConsumptionManyData(data, countOfRecords);

				//success, increase count 
				count++;
			}

			return count;
		}

		public int TestReadOneMasterAndManyDetails(int countOfRecords)
		{
			int count = 0;
			this.TimeStart();
			while (!this.IsTimeUp())
			{
				object data = _server.FetchOrderAndLines(this.GetRandomId(), countOfRecords);
				
				//consumption the returned data
				this.ConsumptionManyData(data, countOfRecords);
				
				//success, increase count
				count++;
			}

			return count;
		}

		private void ConsumptionManyData(object data, int expectCountOfRecord)
		{
			int productId;
			decimal priceForEach;
			int noOfItems;
			string comment; 

			DataSet ds = data as DataSet;
			if (ds != null)
			{
				System.Diagnostics.Trace.Assert(ds.Tables[(int)OrderTables.OrderLines].Rows.Count == expectCountOfRecord, "wrong data count");

				foreach (DataRow r in ds.Tables[(int)OrderTables.OrderLines].Rows )
				{
					productId = (int)r[(int)OrderLineColumns.ProductId];
					priceForEach = (decimal)r[(int)OrderLineColumns.PriceForEach];
					noOfItems = (int)r[(int)OrderLineColumns.NoOfItems];
					comment = r[(int)OrderLineColumns.Comment].ToString();
				}

			}

			Order myOrder = data as Order;
			if (myOrder != null)
			{
				System.Diagnostics.Trace.Assert(myOrder.Lines.Count == expectCountOfRecord, "wrong data count");

				foreach (OrderLine ol in myOrder.Lines)
				{
					productId = ol.ProductId;
					priceForEach = ol.PriceForEach;
					noOfItems = ol.NoOfItems;
					comment = ol.Comment;
				}
			}
		}
	}
}
